// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SubSceneCameraAlign.hpp"

using namespace Dawn;

SubSceneCameraAlign::SubSceneCameraAlign(SceneItem *i) : SceneItemComponent(i) {

}

void SubSceneCameraAlign::onRenderTargetResize(
  RenderTarget *target, float_t w, float_t h
) {
  this->realign();
}

void SubSceneCameraAlign::onCameraResize(float_t w, float_t h) {
  this->realign();
}

void SubSceneCameraAlign::realign() {
  float_t diff;
  if(this->camera == nullptr) return;
  if(this->renderTarget == nullptr) return;

  float_t ratio = this->renderTarget->getWidth() / this->renderTarget->getHeight();
  float_t myRatio = this->camera->getRenderTarget()->getWidth() / this->camera->getRenderTarget()->getHeight();
  
  this->camera->type = CAMERA_TYPE_ORTHONOGRAPHIC;
  this->camera->transform->lookAt(glm::vec3(0, 0, 10), glm::vec3(0, 0, 0));

  if(ratio > myRatio) {
    // My Ratio is narrower
    this->camera->orthoLeft = 0;
    this->camera->orthoRight = this->renderTarget->getWidth();
    diff = (this->renderTarget->getHeight() - (this->renderTarget->getWidth() / myRatio)) / 2.0f;
    this->camera->orthoTop = this->renderTarget->getHeight() - diff;
    this->camera->orthoBottom = diff;
  } else {
    // My ratio is wider
    this->camera->orthoBottom = 0;
    this->camera->orthoTop = this->renderTarget->getHeight();
    diff = (this->renderTarget->getWidth() - (this->renderTarget->getHeight() * myRatio)) / 2.0f;
    this->camera->orthoLeft = diff;
    this->camera->orthoRight = this->renderTarget->getWidth() - diff;
  }
}

void SubSceneCameraAlign::setRenderTarget(TextureRenderTarget *renderTarget) {
  assertTrue(this->renderTarget != renderTarget);

  if(this->renderTarget != nullptr) {
    this->renderTarget->eventRenderTargetResized.removeListener(
      this, &SubSceneCameraAlign::onRenderTargetResize
    );
  }

  this->renderTarget = renderTarget;
  this->realign();
  
  if(this->renderTarget != nullptr) {
    this->renderTarget->eventRenderTargetResized.addListener(
      this, &SubSceneCameraAlign::onRenderTargetResize
    );
  }
}

void SubSceneCameraAlign::setCamera(Camera *camera) {
  assertTrue(this->camera != camera);

  if(this->camera != nullptr) {
    this->camera->eventRenderTargetResized.removeListener(
      this, &SubSceneCameraAlign::onCameraResize
    );
  }

  this->camera = camera;
  this->realign();

  if(this->camera != nullptr) {
    this->camera->eventRenderTargetResized.addListener(
      this, &SubSceneCameraAlign::onCameraResize
    );
  }
}

void SubSceneCameraAlign::onStart() {
  this->realign();
}

void SubSceneCameraAlign::onDispose() {
  if(this->renderTarget != nullptr) {
    this->renderTarget->eventRenderTargetResized.removeListener(
      this, &SubSceneCameraAlign::onRenderTargetResize
    );
  }
  if(this->camera != nullptr) {
    this->camera->eventRenderTargetResized.removeListener(
      this, &SubSceneCameraAlign::onCameraResize
    );
  }
}